//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "MessageDialog.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------
// internal
//---------------------------------------------------------------------------
namespace {
const TMsgDlgBtn BUTTON_TABLE[] = {mbYes, mbNo, mbOK, mbAbort, mbRetry,
	mbIgnore, mbAll, mbNoToAll, mbYesToAll};
const int MIN_BUTTON_COUNT = 1;
const int MAX_BUTTON_COUNT = sizeof(BUTTON_TABLE) / sizeof(TMsgDlgBtn);
//---------------------------------------------------------------------------
void __fastcall EnsureButtonCount(int AButtonCount)
{
	if(AButtonCount < MIN_BUTTON_COUNT || AButtonCount > MAX_BUTTON_COUNT) {
		throw Exception("The button count is invalid: " +
			IntToStr(AButtonCount));
	}
}
//---------------------------------------------------------------------------
TMsgDlgButtons __fastcall CountToButtons(int ACount)
{
	TMsgDlgButtons buttons;

	for(int i = 0; i < ACount; i++) {
		buttons << BUTTON_TABLE[i];
	}

	return buttons;
}
//---------------------------------------------------------------------------
int __fastcall CompareButton(void* AItem1, void* AItem2)
{
	return
		static_cast<TButton*>(AItem1)->TabOrder -
		static_cast<TButton*>(AItem2)->TabOrder;
}
//---------------------------------------------------------------------------
TList* __fastcall MakeButtonList(const TForm* AForm)
{
	TList* list = new TList();
	TForm* form = const_cast<TForm*>(AForm);
	TControl* control;

	for(int i = 0; i < form->ControlCount; i++) {
		control = form->Controls[i];
		if(control->InheritsFrom(__classid(TButton))) {
			list->Add(control);
		}
	}
	list->Sort(CompareButton);

	return list;
}
//---------------------------------------------------------------------------
void __fastcall EnableCloseButton(TForm* AForm, bool AEnabled)
{
	EnableMenuItem(
		GetSystemMenu(AForm->Handle, FALSE),
		SC_CLOSE,
		MF_BYCOMMAND | (AEnabled ? MF_ENABLED : MF_GRAYED));
}
//---------------------------------------------------------------------------
void __fastcall InitIcon(TForm* AForm, const TIcon* AIcon)
{
	TControl* control;

	for(int i = 0; i < AForm->ControlCount; i++) {
		control = AForm->Controls[i];
		if(control->InheritsFrom(__classid(TImage))) {
			dynamic_cast<TImage*>(control)->Picture->Icon =
				const_cast<TIcon*>(AIcon);
			break;
		}
	}
}
//---------------------------------------------------------------------------
TStrings* __fastcall MakeCaptionList(AnsiString ACaptions, int AMinCount)
{
	TStringList* list = new TStringList();

	list->Text = ACaptions;
	while(list->Count < AMinCount) {
		list->Add("");
	}

	return list;
}
//---------------------------------------------------------------------------
void __fastcall InitButtons(TForm* AForm, TList* AButtons, int AActive,
	int ACancel, AnsiString ACaptions)
{
	TButton* button;
	TStrings* captions = MakeCaptionList(ACaptions, AButtons->Count);

	for(int i = 0; i < AButtons->Count; i++) {
		button = static_cast<TButton*>(AButtons->Items[i]);
		button->Caption = captions->Strings[i];
		if(i == AActive) {
			AForm->ActiveControl = button;
		}
		button->Cancel = i == ACancel;
		if(button->Cancel) {
			button->ModalResult = mrCancel;
		}
	}

	delete captions;
}
//---------------------------------------------------------------------------
int __fastcall ModalResultToResult(TModalResult AModalResult, TList* AButtons)
{
	TButton* button;

	for(int i = 0; i < AButtons->Count; i++) {
		button = static_cast<TButton*>(AButtons->Items[i]);
		if(AModalResult == button->ModalResult) {
			return i;
		}
	}

	return -1;
}
} // end of namespace
//---------------------------------------------------------------------------
// external
//---------------------------------------------------------------------------
// [Name]
// ShowMessageDialog
// [Function]
// Shows a modal dialog with its icon and buttons customized.
// [Papameters]
// AnsiString ACaption: Caption.
// AnsiString AMessage: Message.
// TMsgDlgType AType: Type.
// const TIcon* AIcon: Icon. Set NULL if no use.
// int AButtonCount: Button count (1 <= n <= 9).
// int AActive: Index of the active button.
// int ACancel: Index of the cancel button.
// AnsiString AButtonCaptions: Button captions separated by line breaks.
//  e.g. "&Yes\r\n&No\r\nYes to &All\r\nCancel"
// [Return Value]
// int: Index of the pressed button.
//---------------------------------------------------------------------------
int __fastcall ShowMessageDialog(AnsiString ACaption, AnsiString AMessage,
	TMsgDlgType AType, const TIcon* AIcon, int AButtonCount, int AActive,
	int ACancel, AnsiString AButtonCaptions)
{
	EnsureButtonCount(AButtonCount);

	int result;
	TForm* dialog = CreateMessageDialog(AMessage, AType,
		CountToButtons(AButtonCount));
	TList* buttons = MakeButtonList(dialog);

	EnableCloseButton(dialog, AButtonCount <= 0 || ACancel >= 0);
	dialog->Font->Charset = DEFAULT_CHARSET;
	dialog->Font->Name = "Default";
	dialog->Caption = ACaption;
	dialog->OnKeyDown = NULL;
	if(AIcon != NULL) {
		InitIcon(dialog, AIcon);
	}
	InitButtons(dialog, buttons, AActive, ACancel, AButtonCaptions);
	result = ModalResultToResult(dialog->ShowModal(), buttons);

	delete buttons;
	delete dialog;

	return result;
}
//---------------------------------------------------------------------------
