//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "MyUtils.h"
#include "TSearchOptions.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------
// control utilities
//---------------------------------------------------------------------------
void __fastcall RedrawMemo(TMemo* AMemo, bool ARedraw)
{
	if(ARedraw) {
		AMemo->Lines->EndUpdate();
		SendMessage(AMemo->Handle, WM_SETREDRAW, ARedraw, 0);
		InvalidateRect(AMemo->Handle, NULL, FALSE);
	}
	else {
		SendMessage(AMemo->Handle, WM_SETREDRAW, ARedraw, 0);
		AMemo->Lines->BeginUpdate();
	}
}
//---------------------------------------------------------------------------
void __fastcall SetMemoText(TMemo* AMemo, AnsiString AText)
{
	try {
		RedrawMemo(AMemo, false);
		SendMessage(AMemo->Handle, EM_SETSEL, 0, -1);
		SendMessage(AMemo->Handle, EM_REPLACESEL, TRUE,
			reinterpret_cast<LPARAM>(AText.c_str()));
		SendMessage(AMemo->Handle, EM_SCROLLCARET, 0, 0);
	}
	__finally {
		RedrawMemo(AMemo, true);
	}
}
//---------------------------------------------------------------------------
void __fastcall SetMemoSelText(TMemo* AMemo, AnsiString ASelText)
{
	try {
		RedrawMemo(AMemo, false);
		SendMessage(AMemo->Handle, EM_REPLACESEL, TRUE,
			reinterpret_cast<LPARAM>(ASelText.c_str()));
		SendMessage(AMemo->Handle, EM_SCROLLCARET, 0, 0);
	}
	__finally {
		RedrawMemo(AMemo, true);
	}
}
//---------------------------------------------------------------------------
bool __fastcall SearchMemo(TMemo* AMemo, const TSearchOptions& AOptions)
{
	bool found = false;
	AnsiString text = AMemo->Lines->Text;
	int start;
	int index = AMemo->SelStart;
	TMemoryStream* buffer = NULL;

	if(AOptions.Operation == TSearchOptions::OPERATION_REPLACE_ALL) {
		start = index = 0;
		buffer = new TMemoryStream();
	}
	for(;;) {
		if(!FindStr(text.c_str(), text.Length(), AOptions.FindText.c_str(),
			AOptions.FindText.Length(), AOptions.MatchCase,
			AOptions.WholeWord, AOptions.Down, index)) {
			break;
		}
		if(AOptions.Operation == TSearchOptions::OPERATION_REPLACE_ALL) {
			found = true;
			buffer->Write(text.c_str() + start, index - start);
			buffer->Write(AOptions.ReplaceText.c_str(),
				AOptions.ReplaceText.Length());
			start = index += AOptions.FindText.Length();
			continue;
		}
		if(AMemo->SelStart == index &&
			AMemo->SelLength == AOptions.FindText.Length()) {
			if(AOptions.Operation == TSearchOptions::OPERATION_FIND) {
				index += AOptions.Down ? AOptions.FindText.Length() :
					-AOptions.FindText.Length();
			}
			else if(AOptions.Operation == TSearchOptions::OPERATION_REPLACE) {
				SetMemoSelText(AMemo, AOptions.ReplaceText);
				text = AMemo->Text;
				index += AOptions.Down ? AOptions.ReplaceText.Length() :
					-AOptions.FindText.Length();
			}
			else {
				break;
			}
			continue;
		}
		found = true;
		AMemo->SelStart = index;
		AMemo->SelLength = AOptions.FindText.Length();
		break;
	}
	if(AOptions.Operation == TSearchOptions::OPERATION_REPLACE_ALL) {
		if(found) {
			buffer->Write(text.c_str() + start, text.Length() - start + 1);
			SetMemoText(AMemo, static_cast<char*>(buffer->Memory));
		}
		delete buffer;
	}

	return found;
}
//---------------------------------------------------------------------------
// filename utilities
//---------------------------------------------------------------------------
AnsiString __fastcall DateTimeToFileName(TDateTime ADateTime)
{
	return ADateTime.FormatString("yyyymmddhhnnss");
}
//---------------------------------------------------------------------------
AnsiString __fastcall ShortFileNameToLongFileName(AnsiString AFileName)
{
	char path[MAX_PATH];
	AnsiString longFileName;

	if(GetLongPathName(AFileName.c_str(), path, MAX_PATH) > 0) {
		longFileName = path;
	}
	else {
		longFileName = AFileName;
	}

	return longFileName;
}
//---------------------------------------------------------------------------
// string utilities
//---------------------------------------------------------------------------
AnsiString __fastcall FormatInt(AnsiString AValue)
{
	static const int group = 3;
	static const char separator = ',';

	AnsiString text = AValue;
	int count = (text.Length() - (*text.c_str() == '-' ? 1 : 0) - 1) / group;
	char *dst, *src;

	text.SetLength(text.Length() + count);
	dst = text.c_str() + text.Length() - 1;
	src = dst - count;
	for(int i = 0; i < count; i++) {
		for(int j = 0; j < group; j++) {
			*dst-- = *src--;
		}
		*dst-- = separator;
	}

	return text;
}
//---------------------------------------------------------------------------
int __fastcall lstrcmpn(const char* AStr1, const char* AStr2, int ALen)
{
	return CompareString(LOCALE_USER_DEFAULT, 0, AStr1, ALen, AStr2,
		ALen) - 2;
}
//---------------------------------------------------------------------------
int __fastcall lstrcmpni(const char* AStr1, const char* AStr2, int ALen)
{
	return CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, AStr1, ALen,
		AStr2, ALen) - 2;
}
//----------------------------------------------------------------------------
bool __fastcall IsAlphaNumeric(const char* AStr)
{
	const int mask = C1_ALPHA | C1_DIGIT;
	WORD results[2];

	if(!GetStringTypeA(LOCALE_USER_DEFAULT, CT_CTYPE1, AStr,
		IsDBCSLeadByte(*AStr) ? 2 : 1, results)) {
		return false;
	}

	return (results[0] & mask) != 0;
}
//----------------------------------------------------------------------------
bool __fastcall IsWholeWord(const char* AText, const char* APos,
	int ALen)
{
	LPCSTR str1, str2;

	for(int i = 0; i < 2; i++) {
		switch(i) {
			case 0:
				str1 = APos;
				str2 = CharPrev(AText, str1);
				break;
			case 1:
				str1 = APos + ALen;
				str2 = CharPrev(AText, str1);
				break;
			default:
				continue;
		}
		if(str1 == str2 || *str1 == '\0' || *str2 == '\0') {
			continue;
		}
		if(IsAlphaNumeric(str1) && IsAlphaNumeric(str2)) {
			return false;
		}
	}

	return true;
}
//---------------------------------------------------------------------------
bool __fastcall FindStr(const char* AStr, int ALen,
	const char* AFindStr, int AFindLen, bool AMatchCase, bool AWholeWord,
	bool ADown, int& AIndex)
{
	const char* curPos = AStr + AIndex;
	const char* lastPos = AStr + ALen - AFindLen;
	int __fastcall (*compare)(const char*, const char*, int) =
		AMatchCase ? lstrcmpn : lstrcmpni;
	bool found = false;

	AIndex = -1;
	while(curPos >= AStr && curPos <= lastPos) {
		if(compare(curPos, AFindStr, AFindLen) == 0 && (!AWholeWord ||
			IsWholeWord(AStr, curPos, AFindLen))) {
			found = true;
			AIndex = curPos - AStr;
			break;
		}
		if(ADown) {
			if(curPos >= lastPos) {
				break;
			}
			curPos = CharNext(curPos);
		}
		else {
			if(curPos <= AStr) {
				break;
			}
			curPos = CharPrev(AStr, curPos);
		}
	}

	return found;
}
//---------------------------------------------------------------------------

