//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <IniFiles.hpp>
#include "TMainForm.h"
#include "THistory.h"
#include "TExam.h"
#include "TExamItems.h"
#include "TExamItem.h"
#include "TOpenDialogs.h"
#include "TStrFindDialog.h"
#include "TEnterDialog.h"
#include "MessageDialog.h"
#include "MyUtils.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm* MainForm = NULL;
//---------------------------------------------------------------------------
namespace
{
const AnsiString INFO_TITLE = "Q-A Card";
const AnsiString INFO_ABOUT = INFO_TITLE +
	" for Windows Version 1.1.0.\r\n\r\n"
	"Copyright (C) 2004 Moochos. All rights reserved.";
const AnsiString FILE_PATH = ExtractFilePath(Application->ExeName);
const AnsiString FILE_HELP = FILE_PATH + "Help\\index.html";
const AnsiString FILE_INI = FILE_PATH + "Settings.ini";
const AnsiString INI_SECTION = "Settings";
const AnsiString INI_KEY = "Values";
const int HISTORY_COUNT = 5;
const int MIN_TAB_WIDTH = 0;
const int MAX_TAB_WIDTH = 96;
} // end of namespace
//---------------------------------------------------------------------------
// __published:
//---------------------------------------------------------------------------
void __fastcall TMainForm::ApplicationIdle(TObject* ASender, bool& ADone)
{
	ADone = true;
	Application->OnIdle = NULL;
	ProcessCommandParams();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormShow(TObject* ASender)
{
	SelectFirst();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCloseQuery(TObject* ASender, bool& ACanClose)
{
	ACanClose = AskClose();
	if(ACanClose) {
		SaveSettings();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::NewM1Click(TObject* ASender)
{
	if(!AskQuitExam()) {
		return;
	}

	ShowStatus("No data");
	NewFile();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OpenM1Click(TObject* ASender)
{
	if(!AskQuitExam()) {
		return;
	}

	AnsiString file;

	if(!SelectFiles(SELECTFILES_OPEN_FILE, file, NULL)) {
		return;
	}
	OpenFile(file);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SaveLogM1Click(TObject* ASender)
{
	if(TellNoLog()) {
		return;
	}

	AnsiString file = "Log" + DateTimeToFileName(Now()) + " - " +
		ExtractFileName(FileName) + ".txt";

	if(!SelectFiles(SELECTFILES_SAVE_LOG, file, NULL)) {
		return;
	}
	try {
		Memo1->Lines->SaveToFile(file);
	}
	catch(...) {
		TellCantSave(file);
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::StartM1Click(TObject* ASender)
{
	if(!AskQuitExam()) {
		return;
	}

	bool review;

	if(ASender == StartM1) {
		review = false;
	}
	else if(ASender == ReviewM1) {
		review = true;
	}
	else {
		return;
	}
	if(TellNoItem(review)) {
		return;
	}
	StartExam(review);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ListM1Click(TObject* ASender)
{
	if(!AskQuitExam()) {
		return;
	}

	ShowStatus("Showed the item list");
	if(IsPlaying()) {
		StopExam();
	}
	Edit1->Text = "";
	Edit2->Text = "";
	Memo1->Text = ItemList;
	Memo1->OnChange(Memo1);
	ActiveControl = Memo1;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::HistoryM1Click(TObject* ASender)
{
	if(!AskQuitExam()) {
		return;
	}

	int index;

	if(ASender == File1M1) {
		index = 0;
	}
	else if(ASender == File2M1) {
		index = 1;
	}
	else if(ASender == File3M1) {
		index = 2;
	}
	else if(ASender == File4M1) {
		index = 3;
	}
	else if(ASender == File5M1) {
		index = 4;
	}
	else {
		return;
	}
	OpenFile(History->Items[index]);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClearM1Click(TObject* ASender)
{
	if(AskOkCancel("Clear the history?") != IDOK) {
		return;
	}
	History->Clear();
	HistoryChange();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitM1Click(TObject* ASender)
{
	Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FindM1Click(TObject* ASender)
{
	TSearchOptions searchOptions;

	if(ASender == FindM1) {
		StrFindDialog->Options = SearchOptions;
		if(!StrFindDialog->Execute(Memo1->SelText)) {
			return;
		}
		SearchOptions = StrFindDialog->Options;
		searchOptions = SearchOptions;
	}
	else if(ASender == FindNextM1) {
		searchOptions = SearchOptions;
	}
	else if(ASender == FindPrevM1) {
		searchOptions = SearchOptions;
		FlipBool(searchOptions.Down);
	}
	else {
		return;
	}
	if(SearchMemo(Memo1, searchOptions)) {
		ActiveControl = Memo1;
	}
	else {
		Application->MessageBox(
			"There is no match.",
			Application->Title.c_str(),
			MB_ICONINFORMATION | MB_SETFOREGROUND);
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SelectAllM1Click(TObject* ASender)
{
	TWinControl* control = ActiveControl;

	if(control != NULL && control->InheritsFrom(__classid(TCustomEdit))) {
		dynamic_cast<TCustomEdit*>(control)->SelectAll();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ViewM1Click(TObject* ASender)
{
	TViewOptions viewOptions = ViewOptions;

	if(ASender == ToolbarM1) {
		FlipBool(viewOptions.ToolBar);
	}
	else if(ASender == ToolbarCaptionsM1) {
		FlipBool(viewOptions.ToolBarCaptions);
	}
	else if(ASender == FontM1) {
		if(!SelectFont(viewOptions)) {
			return;
		}
	}
	else if(ASender == TabWidthM1) {
		if(!EnterTabWidth(viewOptions)) {
			return;
		}
	}
	else if(ASender == SwapPanelsM1) {
		FlipBool(viewOptions.SwapPanels);
	}
	else if(ASender == StatusBarM1) {
		FlipBool(viewOptions.StatusBar);
	}
	else if(ASender == LastPositionM1) {
		FlipBool(viewOptions.LastPosition);
	}
	else if(ASender == TopmostM1) {
		FlipBool(viewOptions.Topmost);
	}
	else if(ASender == DefaultM1) {
		if(AskOkCancel("Set default settings?") != IDOK) {
			return;
		}
		viewOptions = TViewOptions();
	}
	else  {
		return;
	}
	ViewOptions = viewOptions;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OptionsM1Click(TObject* ASender)
{
	TExamOptions examOptions = ExamOptions;

	if(ASender == ShuffleM1) {
		FlipBool(examOptions.Shuffle);
	}
	else if(ASender == SwapM1) {
		FlipBool(examOptions.Swap);
	}
	else if(ASender == CaseSensitiveM1) {
		FlipBool(examOptions.CaseSensitive);
	}
	else if(ASender == DefaultM2) {
		if(AskOkCancel("Set default settings?") != IDOK) {
			return;
		}
		examOptions = TExamOptions();
	}
	else {
		return;
	}
	ExamOptions = examOptions;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::HelpM2Click(TObject* ASender)
{
	if(AskOkCancel("Open the help document in HTML format?") != IDOK) {
		return;
	}
	if(reinterpret_cast<unsigned int>(ShellExecute(Application->Handle,
		"open", FILE_HELP.c_str(), NULL, NULL, SW_SHOWNORMAL)) <= 32) {
		TellCantOpen(FILE_HELP);
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AboutM1Click(TObject* ASender)
{
	ShowMessageDialog(
		"About",
		INFO_ABOUT,
		mtInformation, Application->Icon,
		1, 0, 0, "OK");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ToolButtonClick(TObject* ASender)
{
	if(ASender == NewT1) {
		NewM1->Click();
	}
	else if(ASender == OpenT1) {
		OpenM1->Click();
	}
	else if(ASender == LogT1) {
		SaveLogM1->Click();
	}
	else if(ASender == StartT1) {
		StartM1->Click();
	}
	else if(ASender == ReviewT1) {
		ReviewM1->Click();
	}
	else if(ASender == ListT1) {
		ListM1->Click();
	}
	else if(ASender == SwapT1) {
		SwapM1->Click();
	}
	else if(ASender == ShuffleT1) {
		ShuffleM1->Click();
	}
	else if(ASender == CaseT1) {
		CaseSensitiveM1->Click();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Edit2KeyDown(TObject* ASender, WORD& AKey,
	TShiftState AShift)
{
	if(AKey == VK_RETURN) {
		if(IsPlaying()) {
			AKey = '\0';
			ScoreExam();
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Memo1Change(TObject* ASender)
{
	bool hasLines = Memo1->Lines->Count > 0;

	SaveLogM1->Enabled = hasLines;
	LogT1->Enabled = SaveLogM1->Enabled;
}
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
void __fastcall TMainForm::FlipBool(bool& AValue)
{
	AValue = !AValue;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EnsureParagraph(TStrings* ALines)
{
	if(ALines->Count > 0) {
		ALines->Add("");
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TellError(AnsiString AMessage)
{
	Application->MessageBox(
		AMessage.c_str(),
		Application->Title.c_str(),
		MB_ICONERROR | MB_SETFOREGROUND);
}
//---------------------------------------------------------------------------
int __fastcall TMainForm::AskOkCancel(AnsiString AMessage)
{
	return Application->MessageBox(
		AMessage.c_str(),
		Application->Title.c_str(),
		MB_OKCANCEL | MB_ICONEXCLAMATION | MB_SETFOREGROUND);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TellCantOpen(AnsiString AFile) const
{
	TellError("The program cannot open the document: " + AFile);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TellCantSave(AnsiString AFile) const
{
	TellError("The program cannot save the document: " + AFile);
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::AskClose() const
{
	return AskQuitExam();
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::TellNoLog() const
{
	bool noLog = Memo1->Lines <= 0;

	if(noLog) {
		TellError("There is no log.");
	}

	return noLog;
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::TellNoItem(bool AReview) const
{
	bool noItem = AReview ? Exam->Review->Count <= 0 : Exam->All->Count <= 0;

	if(noItem) {
		TellError("There is no item.");
	}

	return noItem;
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::AskQuitExam() const
{
	bool quitExam = true;

	if(IsPlaying() && AskOkCancel("Quit the current examination?") != IDOK) {
		quitExam = false;
	}

	return quitExam;
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::AskFileType(AnsiString AFile, int& AFileType) const
{
	switch(ShowMessageDialog(
		Application->Title,
		"Select the file type: " + AFile,
		mtWarning, NULL,
		3, 0, 2, "&CSV\r\n&Text\r\nCancel")) {
		case 0:
			AFileType = TExamItems::FILETYPE_CSV;
			return true;
		case 1:
			AFileType = TExamItems::FILETYPE_TEXT;
			return true;
		default:
			return false;
	}
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::SelectFiles(int AOperation, AnsiString& AFileName,
	TStrings* AFiles) const
{
	bool ok = false;
	TOpenDialog* openDialog = OpenDialogs->FindItem(AOperation);

	openDialog->Files->Clear();
	openDialog->FileName = AFileName;
	if(openDialog->Execute()) {
		ok = true;
		AFileName = openDialog->FileName;
		if(AFiles != NULL) {
			AFiles->Assign(openDialog->Files);
		}
		openDialog->InitialDir = ExtractFileDir(AFileName);
		for(int i = 0; i < OpenDialogs->Count; i++) {
			OpenDialogs->Items[i]->InitialDir = openDialog->InitialDir;
		}
	}

	return ok;
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::SelectFont(TViewOptions& AViewOptions) const
{
	bool ok = false;

	FontDialog1->Font->Name = AViewOptions.FontName;
	FontDialog1->Font->Size = AViewOptions.FontSize;
	FontDialog1->Font->Style = AViewOptions.FontStyle;
	if(FontDialog1->Execute()) {
		ok = true;
		AViewOptions.FontName = FontDialog1->Font->Name;
		AViewOptions.FontSize = FontDialog1->Font->Size;
		AViewOptions.FontStyle = FontDialog1->Font->Style;
	}

	return ok;
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::EnterTabWidth(TViewOptions& AViewOptions) const
{
	bool ok = false;
	AnsiString value = AViewOptions.TabWidth;
	int tabWidth;

	while(EnterDialog->Execute("Tab Width", "Enter the tab width.", value)) {
		if(!TryStrToInt(value, tabWidth) || tabWidth < MIN_TAB_WIDTH ||
			tabWidth > MAX_TAB_WIDTH) {
			TellError("Set the number between " + IntToStr(MIN_TAB_WIDTH) +
				" and " + IntToStr(MAX_TAB_WIDTH) + ": " + value);
			continue;
		}
		ok = true;
		AViewOptions.TabWidth = tabWidth;
		break;
	}

	return ok;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::NewFile()
{
	FileName = "";
	Exam->Clear();
	ExamChange();
	Edit1->Text = "";
	Edit2->Text = "";
	Memo1->Text = "";
	Memo1->OnChange(Memo1);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OpenFile(AnsiString AFile)
{
	int fileType;
	TExamItems* items;

	if(!OpenFile_GetFileType(AFile, fileType)) {
		return;
	}
	items = new TExamItems();
	if(items->LoadFromFile(AFile, fileType)) {
		NewFile();
		FileName = AFile;
		Exam->All->Assign(items);
		ExamChange();
	}
	else {
		TellCantOpen(AFile);
	}

	delete items;
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::OpenFile_GetFileType(AnsiString AFile,
	int& AFileType)
{
	bool ok = true;
	AnsiString ext = ExtractFileExt(AFile);

	if(SameFileName(ext, ".csv")) {
		AFileType = TExamItems::FILETYPE_CSV;
	}
	else if(SameFileName(ext, ".txt")) {
		AFileType = TExamItems::FILETYPE_TEXT;
	}
	else {
		if(!AskFileType(AFile, AFileType)) {
			ok = false;
		}
	}

	return ok;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::StartExam(bool AReview)
{
	ShowStatus("Started");
	Exam->Reset();
	if(AReview) {
		Exam->Rest->Assign(Exam->Review);
	}
	else {
		Exam->Rest->Assign(Exam->All);
		Exam->Review->Clear();
	}
	ExamChange();
	Edit1->Text = "";
	Edit2->Text = "";
	Memo1->Text = "";
	Memo1->OnChange(Memo1);

	ProceedExam();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ProceedExam()
{
	{
		int answered = Exam->Hit->Count + Exam->Miss->Count;
		int total = Exam->Rest->Count + answered;

		ShowStatus("Answered " + FormatInt(answered) + "/" +
			FormatInt(total) + " items");
	}
	if(Exam->Rest->Count <= 0) {
		EndExam();
		return;
	}
	Exam->CurrentOptions = ExamOptions;
	if(Exam->CurrentOptions.Shuffle) {
		Exam->CurrentIndex = random(Exam->Rest->Count);
	}
	else {
		Exam->CurrentIndex = 0;
	}
	ExamChange();
	if(Exam->CurrentOptions.Swap) {
		Edit1->Text = Exam->Rest->Items[Exam->CurrentIndex]->Answers[0];
	}
	else {
		Edit1->Text = Exam->Rest->Items[Exam->CurrentIndex]->Question;
	}
	Edit2->Text = "";
	ActiveControl = Edit2;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ScoreExam()
{
	AnsiString question;
	TStringList* answers = new TStringList();
	AnsiString entry;

	ScoreExam_Prepare(question, answers, entry);
	ScoreExam_Score(question, answers, entry);

	ProceedExam();

	delete answers;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ScoreExam_Prepare(AnsiString& AQuestion,
	TStringList* AAnswers, AnsiString& AEntry) const
{
	TExamItem* item = Exam->Rest->Items[Exam->CurrentIndex];

	if(Exam->CurrentOptions.Swap) {
		AQuestion = item->Answers[0];
		AAnswers->Add(item->Question);
	}
	else {
		AQuestion = item->Question;
		for(int i = 0; i < item->AnswerCount; i++) {
			AAnswers->Add(item->Answers[i]);
		}
	}
	AEntry = Edit2->Text.Trim();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ScoreExam_Score(AnsiString& AQuestion,
	TStringList* AAnswers, AnsiString& AEntry)
{
	AnsiString entry = AEntry.Trim();
	bool __fastcall (*match)(const AnsiString, const AnsiString);
	bool hit = false;

	if(Exam->CurrentOptions.CaseSensitive) {
		match = AnsiSameStr;
	}
	else {
		match = AnsiSameText;
	}
	for(int i = 0; i < AAnswers->Count; i++) {
		if(match(entry, AAnswers->Strings[i].Trim())) {
			hit = true;
			break;
		}
	}
	if(hit) {
		Exam->Rest->MoveTo(Exam->CurrentIndex, Exam->Hit);
	}
	else {
		Exam->Rest->MoveTo(Exam->CurrentIndex, Exam->Miss);
		EnsureParagraph(Memo1->Lines);
		Memo1->Lines->Add("[Miss " + FormatInt(Exam->Miss->Count + 1) + "]");
		Memo1->Lines->Add("Question: " + AQuestion);
		for(int i = 0; i < AAnswers->Count; i++) {
			Memo1->Lines->Add("Answer " + IntToStr(i + 1) + ": " +
				AAnswers->Strings[i]);
		}
		Memo1->Lines->Add("Entry: " + AEntry);
		Memo1->OnChange(Memo1);
	}
	ExamChange();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::StopExam()
{
	Exam->Reset();
	ExamChange();
	Edit1->Text = "";
	Edit2->Text = "";
	Memo1->Text = "";
	Memo1->OnChange(Memo1);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EndExam()
{
	int total = (Exam->Hit->Count + Exam->Miss->Count);
	int ratio = total != 0 ? Exam->Hit->Count * 100 / total : 0;

	Edit1->Text = "";
	Edit2->Text = "";
	EnsureParagraph(Memo1->Lines);
	Memo1->Lines->Add("[Results]");
	Memo1->Lines->Add("Hit: " + FormatInt(Exam->Hit->Count));
	Memo1->Lines->Add("Miss: " + FormatInt(Exam->Miss->Count));
	Memo1->Lines->Add("Ratio: " + FormatInt(ratio) + "%");
	Memo1->OnChange(Memo1);
	ActiveControl = Memo1;
	Exam->Review->Assign(Exam->Miss);
	Exam->Reset();
	ExamChange();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ProcessCommandParams()
{
	if(ParamCount() <= 0) {
		return;
	}

	TStringList* files = new TStringList();

	for(int i = 0; i < ParamCount(); i++) {
		files->Add(ShortFileNameToLongFileName(ParamStr(i + 1)));
	}
	FilesDrop(files);

	delete files;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::LoadSettings()
{
	if(!FileExists(FILE_INI)) {
		return;
	}

	TMemIniFile* ini = new TMemIniFile(FILE_INI);
	TStringList* params = new TStringList();
	int index = 0;

	try {
		params->CommaText = ini->ReadString(INI_SECTION, INI_KEY, "");
		{
			History->CommaText = params->Strings[index++];
			HistoryChange();
		}
		{
			TViewOptions viewOptions = ViewOptions;

			viewOptions.ToolBar = StrToBool(params->Strings[index++]);
			viewOptions.ToolBarCaptions = StrToBool(params->Strings[index++]);
			viewOptions.FontName = params->Strings[index++];
			viewOptions.FontSize = StrToInt(params->Strings[index++]);
			viewOptions.FontStyleAsInt = StrToInt(params->Strings[index++]);
			viewOptions.TabWidth = StrToInt(params->Strings[index++]);
			viewOptions.SwapPanels = StrToBool(params->Strings[index++]);
			viewOptions.StatusBar = StrToBool(params->Strings[index++]);
			viewOptions.LastPosition = StrToBool(params->Strings[index++]);
			viewOptions.Topmost = StrToBool(params->Strings[index++]);
			ViewOptions = viewOptions;
		}
		{
			TExamOptions examOptions = ExamOptions;

			examOptions.Shuffle = StrToBool(params->Strings[index++]);
			examOptions.Swap = StrToBool(params->Strings[index++]);
			examOptions.CaseSensitive = StrToBool(params->Strings[index++]);
			ExamOptions = examOptions;
		}
		{
			TWindowState windowState;
			TRect boundsRect;

			windowState = static_cast<TWindowState>(
				StrToInt(params->Strings[index++]));
			boundsRect.left = StrToInt(params->Strings[index++]);
			boundsRect.top = StrToInt(params->Strings[index++]);
			boundsRect.right = StrToInt(params->Strings[index++]);
			boundsRect.bottom = StrToInt(params->Strings[index++]);
			if(windowState == wsNormal && ViewOptions.LastPosition) {
				BoundsRect = boundsRect;
			}
			for(int i = 0; i < OpenDialogs->Count; i++) {
				OpenDialogs->Items[i]->InitialDir =
					params->Strings[index++];
				OpenDialogs->Items[i]->FilterIndex =
					StrToInt(params->Strings[index++]);
			}
		}
	}
	catch(...) {
		TellCantOpen(FILE_INI);
	}

	delete params;
	delete ini;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SaveSettings()
{
	TMemIniFile* ini = new TMemIniFile(FILE_INI);
	TStringList* params = new TStringList();

	try {
		{
			params->Add(History->CommaText);
		}
		{
			TViewOptions viewOptions = ViewOptions;

			params->Add(BoolToStr(viewOptions.ToolBar));
			params->Add(BoolToStr(viewOptions.ToolBarCaptions));
			params->Add(viewOptions.FontName);
			params->Add(viewOptions.FontSize);
			params->Add(viewOptions.FontStyleAsInt);
			params->Add(viewOptions.TabWidth);
			params->Add(BoolToStr(viewOptions.SwapPanels));
			params->Add(BoolToStr(viewOptions.StatusBar));
			params->Add(BoolToStr(viewOptions.LastPosition));
			params->Add(BoolToStr(viewOptions.Topmost));
		}
		{
			TExamOptions examOptions = ExamOptions;

			params->Add(BoolToStr(examOptions.Shuffle));
			params->Add(BoolToStr(examOptions.Swap));
			params->Add(BoolToStr(examOptions.CaseSensitive));
		}
		{
			TRect boundsRect = BoundsRect;

			params->Add(WindowState);
			params->Add(BoundsRect.left);
			params->Add(BoundsRect.top);
			params->Add(BoundsRect.right);
			params->Add(BoundsRect.bottom);
			for(int i = 0; i < OpenDialogs->Count; i++) {
				params->Add(OpenDialogs->Items[i]->InitialDir);
				params->Add(OpenDialogs->Items[i]->FilterIndex);
			}
			params->Add("");
		}
		ini->WriteString(INI_SECTION, INI_KEY, params->CommaText);
		ini->UpdateFile();
	}
	catch(...) {
		TellCantSave(FILE_INI);
	}

	delete params;
	delete ini;
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetFileName(AnsiString AFileName)
{
	if(AFileName != FFileName) {
		FFileName = AFileName;
		FileNameChange();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileNameChange()
{
	AnsiString title;

	if(FileName.IsEmpty()) {
		title = INFO_TITLE;
	}
	else {
		title = ExtractFileName(FileName) + " - " + INFO_TITLE;
		History->Add(FileName);
		HistoryChange();
	}
	Application->Title = title;
	Caption = title;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TMainForm::GetItemList() const
{
	AnsiString itemList;
	TStringList* lines = new TStringList();
	TExamItem* item;

	lines->Add("[Information]");
	lines->Add("Filename: " + FileName);
	lines->Add("Item count: " + FormatInt(Exam->All->Count));
	for(int i = 0; i < Exam->All->Count; i++) {
		EnsureParagraph(lines);
		item = Exam->All->Items[i];
		lines->Add("[Item " + FormatInt(i + 1) + "]");
		lines->Add("Question: " + item->Question);
		for(int j = 0; j < item->AnswerCount; j++) {
			lines->Add("Answer " + IntToStr(j + 1) + ": " + item->Answers[j]);
		}
	}
	itemList = lines->Text;

	delete lines;

	return itemList;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetSearchOptions(TSearchOptions ASearchOptions)
{
	if(ASearchOptions != FSearchOptions) {
		FSearchOptions = ASearchOptions;
		SearchOptionsChange();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SearchOptionsChange()
{
	bool canFindMore =
		(FSearchOptions.Operation == TSearchOptions::OPERATION_FIND) &&
		!FSearchOptions.FindText.IsEmpty();

	FindNextM1->Enabled = canFindMore;
	FindPrevM1->Enabled = canFindMore;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetViewOptions(TViewOptions AViewOptions)
{
	if(AViewOptions != FViewOptions) {
		FViewOptions = AViewOptions;
		ViewOptionsChange();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ViewOptionsChange()
{
	ToolBar1->Visible = FViewOptions.ToolBar;
	ToolBar1->ShowCaptions = FViewOptions.ToolBarCaptions;
	ToolBar1->ShowHint = !ToolBar1->ShowCaptions;
	if(!ToolBar1->ShowCaptions) {
		ToolBar1->ButtonWidth = 22;
		ToolBar1->ButtonHeight = 21;
	}
	{
		int count = 3;
		TFont* font;
		HWND handle;
		DWORD tabWidth = 4 * FViewOptions.TabWidth;

		for(int i = 0; i < count; i++) {
			switch(i) {
				case 0: font = Edit1->Font; handle = Edit1->Handle; break;
				case 1: font = Edit2->Font; handle = Edit2->Handle; break;
				case 2: font = Memo1->Font; handle = Memo1->Handle; break;
				default: continue;
			}
			font->Name = FViewOptions.FontName;
			font->Size = FViewOptions.FontSize;
			font->Style = FViewOptions.FontStyle;
			SendMessage(handle, EM_SETTABSTOPS, 1,
				reinterpret_cast<LPARAM>(&tabWidth));
		}
	}
	Panel2->Align = FViewOptions.SwapPanels ? alBottom : alTop;
	StatusBar1->Visible = FViewOptions.StatusBar;
	SetWindowPos(Handle, FViewOptions.Topmost ? HWND_TOPMOST : HWND_NOTOPMOST,
		0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);

	ToolbarM1->Checked = FViewOptions.ToolBar;
	ToolbarCaptionsM1->Checked = FViewOptions.ToolBarCaptions;
	SwapPanelsM1->Checked = FViewOptions.SwapPanels;
	StatusBarM1->Checked = FViewOptions.StatusBar;
	LastPositionM1->Checked = FViewOptions.LastPosition;
	TopmostM1->Checked = FViewOptions.Topmost;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetExamOptions(TExamOptions AExamOptions)
{
	if(AExamOptions != FExamOptions) {
		FExamOptions = AExamOptions;
		ExamOptionsChange();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExamOptionsChange()
{
	ShuffleM1->Checked = FExamOptions.Shuffle;
	SwapM1->Checked = FExamOptions.Swap;
	CaseSensitiveM1->Checked = FExamOptions.CaseSensitive;
	ShuffleT1->Down = ShuffleM1->Checked;
	SwapT1->Down = SwapM1->Checked;
	CaseT1->Down = CaseSensitiveM1->Checked;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::HistoryChange()
{
	TMenuItem* menuItems[] = {File1M1, File2M1, File3M1, File4M1, File5M1};
	AnsiString file;
	bool enabled;
	bool canClear = false;

	for(int i = 0; i < History->Count; i++) {
		file = ExtractFileName(History->Items[i]);
		if(file.IsEmpty()) {
			file = "(None)";
			enabled = false;
		}
		else {
			enabled = true;
			canClear = true;
		}
		menuItems[i]->Caption = file + " (&" + IntToStr(i + 1) + ")";
		menuItems[i]->Enabled = enabled;
	}
	ClearM1->Enabled = canClear;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExamChange()
{
	bool hasData = Exam->All->Count > 0;
	bool hasReview = Exam->Review->Count > 0;
	bool hasRest = Exam->Rest->Count > 0;

	StartM1->Enabled = hasData;
	ReviewM1->Enabled = hasReview;
	ListM1->Enabled = hasData;
	StartT1->Enabled = StartM1->Enabled;
	ReviewT1->Enabled = ReviewM1->Enabled;
	ListT1->Enabled = ListM1->Enabled;
	Edit2->ReadOnly = !hasRest;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FilesDrop(const TStrings* AFiles)
{
	if(!AskQuitExam()) {
		return;
	}

	TStrings* files = const_cast<TStrings*>(AFiles);

	if(files->Count <= 0) {
		return;
	}

	ShowStatus(FormatInt(files->Count) + " items are dropped");
	if(files->Count == 1) {
		OpenFile(files->Strings[0]);
	}
	else {
		TellError("Set one file to open.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMDropFiles(TWMDropFiles& AMessage)
{
	HDROP handle = reinterpret_cast<HDROP>(AMessage.Drop);
	int count = DragQueryFile(handle, 0xFFFFFFFF, NULL, 0);
	char temp[MAX_PATH];
	TStringList* files = new TStringList();

	SetForegroundWindow(Application->Handle);

	for(int i = 0; i < count; i++) {
		if(DragQueryFile(handle, i, temp, MAX_PATH) > 0) {
			files->Add(ShortFileNameToLongFileName(temp));
		}
	}
	DragFinish(handle);

	if(files->Count > 0) {
		FilesDrop(files);
	}

	delete files;
}
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* AOwner) :
	TForm(AOwner),
	FFileName(),
	FHistory(new THistory()),
	FExam(new TExam()),
	FViewOptions(),
	FExamOptions(),
	FOpenDialogs(new TOpenDialogs())
{
	randomize();
	Application->Title = INFO_TITLE;
	Application->OnIdle = ApplicationIdle;
	Caption = INFO_TITLE;
	DragAcceptFiles(Handle, true);
	Memo1->OnChange(Memo1);
	FileNameChange();
	History->Count = HISTORY_COUNT;
	HistoryChange();
	ExamChange();
	SearchOptionsChange();
	ViewOptionsChange();
	ExamOptionsChange();
	OpenDialogs->Add(SELECTFILES_OPEN_FILE, OpenDialog1);
	OpenDialogs->Add(SELECTFILES_SAVE_LOG, SaveDialog1);
	LoadSettings();
	ShowStatus("Ready");
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
	delete FOpenDialogs; FOpenDialogs = NULL;
	delete FExam; FExam = NULL;
	delete FHistory; FHistory = NULL;
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::IsPlaying() const
{
	return Exam->Rest->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ShowStatus(AnsiString AStatus)
{
	StatusBar1->SimpleText = AStatus;
}
//---------------------------------------------------------------------------

