//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "TSearchDialog.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TSearchDialog* SearchDialog = NULL;
//---------------------------------------------------------------------------
// class TSearchDialog : public TForm
//---------------------------------------------------------------------------
// __published:
//---------------------------------------------------------------------------
void __fastcall TSearchDialog::FormShow(TObject* ASender)
{
	SelectFirst();
}
//---------------------------------------------------------------------------
void __fastcall TSearchDialog::FormCloseQuery(TObject* ASender,
	bool& ACanClose)
{
	ACanClose = true;
	switch(ModalResult) {
		case mrYes:
		case mrNo:
		case mrAll:
			if(!UpdateParams()) {
				ACanClose = false;
			}
			break;
		default:
			RestoreParams();
			break;
	}
}
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
int __fastcall TSearchDialog::ModalResultToOperation(TModalResult AModalResult)
{
	switch(AModalResult) {
		case mrYes:
			return TSearchOptions::OPERATION_FIND;
		case mrNo:
			return TSearchOptions::OPERATION_REPLACE;
		case mrAll:
			return TSearchOptions::OPERATION_REPLACE_ALL;
		default:
			throw 0;
	}
}
//---------------------------------------------------------------------------
void __fastcall TSearchDialog::TellError(TWinControl* AErrorControl)
{
	AnsiString errorMessage;

	ActiveControl = AErrorControl;
	if(AErrorControl == Edit1) {
		errorMessage = "Set the text to find.";
	}
	else {
		errorMessage = "Set the proper value.";
	}
	Application->MessageBox(
		errorMessage.c_str(),
		Application->Title.c_str(),
		MB_ICONERROR | MB_SETFOREGROUND);
}
//---------------------------------------------------------------------------
void __fastcall TSearchDialog::ResetParams()
{
	Options = TSearchOptions();
}
//---------------------------------------------------------------------------
void __fastcall TSearchDialog::BackupParams(AnsiString AFindText)
{
	FBackup.Options = Options;
	if(!AFindText.IsEmpty()) {
		TSearchOptions options = Options;

		options.FindText = AFindText;
		Options = options;
	}
}
//---------------------------------------------------------------------------
void __fastcall TSearchDialog::RestoreParams()
{
	Options = FBackup.Options;
}
//---------------------------------------------------------------------------
bool __fastcall TSearchDialog::UpdateParams()
{
	try {
		TSearchOptions options;

		options.Operation = ModalResultToOperation(ModalResult);
		options.FindText = Edit1->Text;
		if(options.FindText.IsEmpty()) {
			TellError(Edit1);
			return false;
		}
		options.ReplaceText = Edit2->Text;
		options.MatchCase = CheckBox1->Checked;
		options.WholeWord = CheckBox2->Checked;
		options.Down = RadioGroup1->ItemIndex == 1;
		Options = options;
	}
	catch(...) {
		TellError(NULL);
		return false;
	}

	return true;
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
void __fastcall TSearchDialog::SetOptions(TSearchOptions AOptions)
{
	FOptions = AOptions;
	Edit1->Text = FOptions.FindText;
	Edit2->Text = FOptions.ReplaceText;
	CheckBox1->Checked = FOptions.MatchCase;
	CheckBox2->Checked = FOptions.WholeWord;
	RadioGroup1->ItemIndex = FOptions.Down ? 1 : 0;
}
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TSearchDialog::TSearchDialog(TComponent* AOwner) :
	TForm(AOwner),
	FOptions()
{
	ResetParams();
}
//---------------------------------------------------------------------------
__fastcall TSearchDialog::~TSearchDialog()
{
	// do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TSearchDialog::Execute(AnsiString AFindText, bool AFind)
{
	bool ok = false;

	BackupParams(AFindText);
	Button1->Default = AFind;
	Button2->Default = !AFind;
	switch(ShowModal()) {
		case mrYes:
		case mrNo:
		case mrAll:
			ok = true;
			break;
	}

	return ok;
}
//---------------------------------------------------------------------------

